package cn.jlsxwkj.common.reader;


import cn.jlsxwkj.common.utils.SplitDocument;
import lombok.Data;
import org.springframework.ai.document.Document;
import org.springframework.ai.document.DocumentReader;
import org.springframework.util.StreamUtils;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.util.List;

/**
 * @author zh
 * 文本读取
 */
@Data
public class ParagraphTextReader implements DocumentReader {

    private final InputStream inputStream;
    private final Charset charset = StandardCharsets.UTF_8;
    private final int windowSize;
    private final String fileName;

    public ParagraphTextReader(byte[] bytes,
                               String fileName,
                               int windowSize) {
        this.inputStream = new ByteArrayInputStream(bytes);
        this.windowSize = windowSize;
        this.fileName = fileName;
    }

    /**
     * 读取文本内容,并根据换行进行分段,采用窗口模式,窗口为段落的数量
     *
     * @return 文档信息列表
     */
    @Override
    public List<Document> get() {
        String document = null;
        try {
            document = StreamUtils.copyToString(inputStream, this.charset);
        } catch (IOException e) {
            e.printStackTrace();
        }
        assert document != null;
        return SplitDocument.splitStringToListDocument(document, this.windowSize, this.fileName);
    }
}