package cn.jlsxwkj.common.utils;

import cn.hutool.core.collection.ListUtil;
import org.springframework.ai.document.Document;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author jay
 */
public class SplitDocument {

    public static final String START_PARAGRAPH_NUMBER = "startParagraphNumber";
    public static final String END_PARAGRAPH_NUMBER = "endParagraphNumber";
    public static final String FILE_NAME = "source";

    /**
     * 分割字符并转换为多个文档
     *
     * @param document   文档
     * @param windowSize 窗口大小
     * @param fileName   文件名
     * @return 多个文档
     */
    public static List<Document> splitStringToListDocument(String document,
                                                           int windowSize,
                                                           String fileName) {
        if (document.isBlank()) {
            return null;
        }
        ArrayList<Document> readDocuments = new ArrayList<>();
        List<String> paragraphs = Arrays.stream(document.split("\n")).distinct().dropWhile(""::equals).collect(Collectors.toList());
        //采用窗口滑动读取
        int startIndex = 0;
        int endIndex = startIndex + windowSize;
        if (endIndex > paragraphs.size()) {
            readDocuments.add(toDocument(paragraphs, fileName, startIndex + 1, paragraphs.size()));
        } else {
            for (; endIndex <= paragraphs.size(); startIndex++, endIndex++) {
                readDocuments.add(toDocument(ListUtil.sub(paragraphs, startIndex, endIndex), fileName, startIndex + 1, endIndex));
            }
        }
        return readDocuments;
    }

    /**
     * 封装段落成文档
     *
     * @param paragraphList     段落内容列表
     * @param startParagraphNum 开始段落编码
     * @param endParagraphNum   结束段落编码
     * @return 文档信息
     */
    private static Document toDocument(List<String> paragraphList,
                                       String fileName,
                                       int startParagraphNum,
                                       int endParagraphNum) {
        Document doc = new Document(String.join("\n", paragraphList));
        doc.getMetadata().put(FILE_NAME, fileName);
        doc.getMetadata().put(START_PARAGRAPH_NUMBER, startParagraphNum);
        doc.getMetadata().put(END_PARAGRAPH_NUMBER, endParagraphNum);
        return doc;
    }

}
