package cn.jlsxwkj.moudles.chat;

import cn.hutool.http.HttpRequest;
import cn.hutool.json.JSONUtil;
import cn.jlsxwkj.common.config.UserConfig;
import cn.jlsxwkj.common.utils.Log;
import cn.jlsxwkj.moudles.chat.message.Message;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Component;
import reactor.core.publisher.Flux;
import reactor.core.scheduler.Scheduler;
import reactor.core.scheduler.Schedulers;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.List;

/**
 * @author zh
 * chat 工具类, 用来请求上游 chat api
 */
@Component
public class ChatUtil {

    @Resource
    private UserConfig userConfig;
    private InputStream inputStream;
    private InputStreamReader inputStreamReader;
    private BufferedReader bufferedReader;
    private final Scheduler scheduler = Schedulers.single();

    /**
     * 请求上游 chat api
     *
     * @param messages 消息列表(带历史对话)
     * @param isStream 是否流式返回
     * @return 流式返回
     */
    public Flux<ChatResponse> chat(List<Message> messages, boolean isStream) {
        ChatRequest chatRequest = new ChatRequest();
        chatRequest.setModel(userConfig.getModel());
        chatRequest.setMessages(messages);
        chatRequest.setStream(isStream);
        Log.info(this.getClass(), JSONUtil.toJsonStr(chatRequest));
        Log.info(this.getClass(), userConfig.getModel());
        Log.info(this.getClass(), userConfig.getBaseUrl());
        inputStream = HttpRequest.post(userConfig.getBaseUrl() + "/api/chat")
                .body(JSONUtil.toJsonStr(chatRequest))
                .execute(isStream)
                .bodyStream();
        inputStreamReader = new InputStreamReader(inputStream);
        bufferedReader = new BufferedReader(inputStreamReader);
        return Flux.fromStream(bufferedReader.lines())
                .map(str -> JSONUtil.parse(str).toBean(ChatResponse.class))
                .subscribeOn(scheduler, false);
    }

    /**
     * 停止流
     */
    public void stop() {
        try {
            bufferedReader.close();
        } catch (IOException e) {
            Log.error(e.getClass(), e.getMessage());
        }
        try {
            inputStreamReader.close();
        } catch (IOException e) {
            Log.error(e.getClass(), e.getMessage());
        }
        try {
            inputStream.close();
        } catch (IOException e) {
            Log.error(e.getClass(), e.getMessage());
        }
    }

    /**
     * 获取调度器
     *
     * @return 调度器
     */
    public Scheduler getScheduler() {
        return scheduler;
    }
}
